Package: eaf
Type: Package
Title: Plots of the Empirical Attainment Function
Version: 2.5.2
Authors@R: c(person("Manuel", "López-Ibáñez", role = c("aut", "cre"),
                              email = "manuel.lopez-ibanez@manchester.ac.uk",
                              comment = c(ORCID = "0000-0001-9974-1295")),
             person("Marco", "Chiarandini", role = "aut"),
             person("Carlos", "Fonseca", role = "aut"),
             person("Luís", "Paquete", role = "aut"),
             person("Thomas", "Stützle", role = "aut"),
             person("Mickaël", "Binois", role = "ctb"))
Description: Computation and visualization of the empirical attainment function (EAF) for the analysis of random sets in multi-criterion optimization. M. López-Ibáñez, L. Paquete, and T. Stützle (2010) <doi:10.1007/978-3-642-02538-9_9>.
Depends: R (>= 3.2)
Imports: modeltools, graphics, grDevices, matrixStats, Rdpack
Suggests: extrafont, testthat (>= 3.0.0), withr, viridisLite, spelling
License: GPL (>= 2)
BugReports: https://github.com/MLopez-Ibanez/eaf/issues
URL: https://mlopez-ibanez.github.io/eaf/,
        https://github.com/MLopez-Ibanez/eaf
LazyLoad: true
LazyData: true
Encoding: UTF-8
RoxygenNote: 7.3.2
SystemRequirements: GNU make, Gnu Scientific Library
RdMacros: Rdpack
Config/testthat/edition: 3
Language: en-GB
NeedsCompilation: yes
Packaged: 2025-03-28 09:29:42 UTC; manu
Author: Manuel López-Ibáñez [aut, cre]
    (<https://orcid.org/0000-0001-9974-1295>),
  Marco Chiarandini [aut],
  Carlos Fonseca [aut],
  Luís Paquete [aut],
  Thomas Stützle [aut],
  Mickaël Binois [ctb]
Maintainer: Manuel López-Ibáñez <manuel.lopez-ibanez@manchester.ac.uk>
Repository: CRAN
Date/Publication: 2025-03-28 10:40:16 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-03-28 12:01:58 UTC; unix
