# Generated by roxygen2: do not edit by hand

S3method(eafplot,default)
S3method(eafplot,formula)
S3method(eafplot,list)
export(attsurf2df)
export(avg_hausdorff_dist)
export(choose_eafdiff)
export(choose_eafdiffplot)
export(eafdiff)
export(eafdiffplot)
export(eafplot)
export(eafs)
export(epsilon_additive)
export(epsilon_mult)
export(filter_dominated)
export(hv_contributions)
export(hypervolume)
export(igd)
export(igd_plus)
export(is_nondominated)
export(largest_eafdiff)
export(normalise)
export(pareto_rank)
export(pdf_crop)
export(read.data.sets)
export(read_datasets)
export(symDifPlot)
export(total_whv_rect)
export(vorobDev)
export(vorobT)
export(whv_hype)
export(whv_rect)
export(write_datasets)
import(graphics)
importFrom(Rdpack,reprompt)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,embedFonts)
importFrom(grDevices,gray)
importFrom(grDevices,pdf)
importFrom(utils,modifyList)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(eaf, .registration = TRUE)
