--------------------------------------------------------------------
   Computation of the Empirical Attainment Function in 2 or 3 Dimensions
                                   
                Carlos Fonseca and Manuel López-Ibáñez
                   <manuel.lopez-ibanez@manchester.ac.uk>
--------------------------------------------------------------------

Contents

    * Introduction
    * Usage
    * License
    * Changelog


------------
Introduction
------------

This program computes the empirical attainment function from a number
of sets of nondominated points. This version is based on the original
code written by Carlos M. Fonseca available at 
http://www.tik.ee.ethz.ch/pisa/.

Relevant literature:

[1] V. Grunert da Fonseca, C. M. Fonseca, and A. Hall. Inferential
    performance assessment of stochastic optimizers and the attainment
    function. In E. Zitzler, K. Deb, L. Thiele, C. C. Coello, and
    D. Corne, editors, Evolutionary Multi-criterion Optimization
    (EMO 2001), volume 1993 of Lecture Notes in Computer Science,
    pages 213-225. Springer Verlag, Berlin, Germany, 2001.



------------
Building
------------

The program has been tested on GNU/Linux using bash shell and a
recent version of GCC (>= 4.2). If you have success or problems using
other systems, please let me know.

The default compilation is done with:

  make eaf

You can optimize for a particular computer architecture with the
option "march=", for example:

  make eaf march=pentium4

Modern versions of GCC support a value of 'march=native' that autodetects
your architecture. See the GCC manual for the names of the
architectures supported by your version of GCC.


-------
Usage
-------

For the remainder options available, see the output of 

  eaf --help


------------
License
------------

This software is Copyright (C) 2006, 2007, 2008, 2009.
Carlos Fonseca and Manuel López-Ibáñez.

This program is free software (software libre); you can redistribute
it and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

IMPORTANT NOTE: Please be aware that the fact that this program is
released as Free Software does not excuse you from scientific
propriety, which obligates you to give appropriate credit! If you
write a scientific paper describing research that made substantive use
of this program, it is your obligation as a scientist to (a) mention
the fashion in which this software was used in the Methods section;
(b) mention the algorithm in the References section. The appropriate
citation is: 

    V. Grunert da Fonseca, C. M. Fonseca, and A. Hall. Inferential
    performance assessment of stochastic optimizers and the attainment
    function. In E. Zitzler, K. Deb, L. Thiele, C. C. Coello, and
    D. Corne, editors, Evolutionary Multi-criterion Optimization
    (EMO 2001), volume 1993 of Lecture Notes in Computer Science,
    pages 213-225. Springer Verlag, Berlin, Germany, 2001.

Moreover, as a personal note, I would appreciate it if you would
email <manuel.lopez-ibanez@manchester.ac.uk> with citations of papers referencing
this work so I can mention them to my funding agent and tenure
committee.

------------
Changelog
------------


